%/*
% * T.C.F.S. v. 1.0beta-linux
% *
% * Authors:	Giuseppe Cattaneo, <cattaneo@udsab.dia.unisa.it>
% *		Giuseppe Persiano, <giuper@udsab.dia.unisa.it>
% *		Andrea Cozzolino, <andcoz@mikonos.dia.unisa.it>
% *		Angelo Celentano, <angcel@mikonos.dia.unisa.it>
% *		Aniello Del Sorbo, <anidel@mikonos.dia.unisa.it>
% *		Ermelindo Mauriello, <ermmau@mikonos.dia.unisa.it>
% *		Raffaele Pisapia, <rafpis@mikonos.dia.unisa.it>
% *
% * Permission to use, copy, and modify this software without fee
% * is hereby granted, provided that this entire notice is included in
% * all copies of any software which is or includes a copy or
% * modification of this software and in all copies of the supporting
% * documentation for such software.
% *
% * This software maybe be used for any purpose provided
% * the above copyright notice is retained.  It is supplied
% * as is, with no warranty expressed or implied.
% *
% * This code derives from: 
% * mount.x by Sun Microsystem, Inc.
% */

%/*
% * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
% * unrestricted use provided that this legend is included on all tape
% * media and as a part of the software program in whole or part.  Users
% * may copy or modify Sun RPC without charge, but are not authorized
% * to license or distribute it to anyone else except as part of a product or
% * program developed by the user or with the express written consent of
% * Sun Microsystems, Inc.
% *
% * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
% * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
% * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
% *
% * Sun RPC is provided with no support and without any obligation on the
% * part of Sun Microsystems, Inc. to assist in its use, correction,
% * modification or enhancement.
% *
% * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
% * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
% * OR ANY PART THEREOF.
% *
% * In no event will Sun Microsystems, Inc. be liable for any lost revenue
% * or profits or other special, indirect and consequential damages, even if
% * Sun has been advised of the possibility of such damages.
% *
% * Sun Microsystems, Inc.
% * 2550 Garcia Avenue
% * Mountain View, California  94043
% */

%/*
% * Copyright (c) 1985, 1990 by Sun Microsystems, Inc.
% */
%
%/* from @(#)mount.x	1.3 91/03/11 TIRPC 1.0 */

/*
 * Protocol description for the mount program
 */

#ifdef RPC_HDR
%#ifndef _rpcsvc_mount_h
%#define _rpcsvc_mount_h
#endif

const MNTPATHLEN = 1024;	/* maximum bytes in a pathname argument */
const MNTNAMLEN = 255;		/* maximum bytes in a name argument */
const FHSIZE = 32;		/* size in bytes of a file handle */

/*
 * The fhandle is the file handle that the server passes to the client.
 * All file operations are done using the file handles to refer to a file
 * or a directory. The file handle can contain whatever information the
 * server needs to distinguish an individual file.
 */
typedef opaque fhandle[FHSIZE];	

/*
 * If a status of zero is returned, the call completed successfully, and 
 * a file handle for the directory follows. A non-zero status indicates
 * some sort of error. The status corresponds with UNIX error numbers.
 */
union fhstatus switch (unsigned fhs_status) {
case 0:
	fhandle fhs_fhandle;
default:
	void;
};

/*
 * The type dirpath is the pathname of a directory
 */
typedef string dirpath<MNTPATHLEN>;

/*
 * The type name is used for arbitrary names (hostnames, groupnames)
 */
typedef string name<MNTNAMLEN>;

/*
 * A list of who has what mounted
 */
typedef struct mountbody *mountlist;
struct mountbody {
	name ml_hostname;
	dirpath ml_directory;
	mountlist ml_next;
};

/*
 * A list of netgroups
 */
typedef struct groupnode *groups;
struct groupnode {
	name gr_name;
	groups gr_next;
};

/*
 * A list of what is exported and to whom
 */
typedef struct exportnode *exports;
struct exportnode {
	dirpath ex_dir;
	groups ex_groups;
	exports ex_next;
};

program MOUNTPROG {
	/*
	 * Version one of the mount protocol communicates with version two
	 * of the NFS protocol. The only connecting point is the fhandle 
	 * structure, which is the same for both protocols.
	 */
	version MOUNTVERS {
		/*
		 * Does no work. It is made available in all RPC services
		 * to allow server reponse testing and timing
		 */
		void
		MOUNTPROC_NULL(void) = 0;

		/*	
		 * If fhs_status is 0, then fhs_fhandle contains the
	 	 * file handle for the directory. This file handle may
		 * be used in the NFS protocol. This procedure also adds
		 * a new entry to the mount list for this client mounting
		 * the directory.
		 * Unix authentication required.
		 */
		fhstatus 
		MOUNTPROC_MNT(dirpath) = 1;

		/*
		 * Returns the list of remotely mounted filesystems. The 
		 * mountlist contains one entry for each hostname and 
		 * directory pair.
		 */
		mountlist
		MOUNTPROC_DUMP(void) = 2;

		/*
		 * Removes the mount list entry for the directory
		 * Unix authentication required.
		 */
		void
		MOUNTPROC_UMNT(dirpath) = 3;

		/*
		 * Removes all of the mount list entries for this client
		 * Unix authentication required.
		 */
		void
		MOUNTPROC_UMNTALL(void) = 4;

		/*
		 * Returns a list of all the exported filesystems, and which
		 * machines are allowed to import it.
		 */
		exports
		MOUNTPROC_EXPORT(void)  = 5;

		/*
		 * Identical to MOUNTPROC_EXPORT above
		 */
		exports
		MOUNTPROC_EXPORTALL(void) = 6;
	} = 1;
} = 100005;

#ifdef RPC_HDR
%#endif /*!_rpcsvc_mount_h*/
#endif

/* Inizio di ADM */
/* error conditions */
enum sfsstat {
	SFS_OK=100,		
	SFSERR_PERM=101,		
	SFSERR_IFULL=102,		
	SFSERR_NOINS=103,		
	SFSERR_EXIST=104,		
	SFSERR_NODIR=105,		
	SFSERR_BADKEY=106,	
	SFSERR_BADNAME=107,	
	ADM_BADUSAGE=200,
	ADM_CWDERR=201,
	ADM_ACCERR=202,
	ADM_FAILED=203
};

const SFS_MAXNAME=1024;

struct sfs_stickargs {
	string local_dir<SFS_MAXNAME>;	/* user sideview exported dir */
	string dummy_dir<SFS_MAXNAME>;	/* for mount purpose */
	string remote_dir<SFS_MAXNAME>; /* mounted via NFS */
	string work_dir<SFS_MAXNAME>;   /* user sideview working dir */
};

struct sfs_unstickargs {
	string name<SFS_MAXNAME>;	/* instance name */
};

struct sfs_lookupargs {
	string name<SFS_MAXNAME>;	/* instance name */
	unsigned mode;
};

struct sfs_lookupres {
	string remote<SFS_MAXNAME>;
	string work<SFS_MAXNAME>;
	string dummy<SFS_MAXNAME>;
};

const SFS_PORT = 5049;

program ADM_PROGRAM {
	version ADM_VERSION {
		void ADMPROC_NULL(void) = 0;
		sfsstat ADMPROC_STICK(sfs_stickargs) = 1;
		sfsstat ADMPROC_UNSTICK(sfs_unstickargs) = 2;
		sfs_lookupres ADMPROC_LOOKUP(sfs_lookupargs) = 3;	
	} = 2;
} = 0x21234567;


















